; SDCC helper for Enterprise computer
; (C)2012 LGB Gábor Lénárt lgb@lgb.hu

        .module crt0
       	.globl	_main
	.area _CODE

	STACK_SIZE = 512
	X_SIZ_VID = 40
	Y_SIZ_VID = 24
	MODE_VID = 0
	COLR_VID = 0

	CHANNEL_VIDEO = 0x66
	CHANNEL_KEYBOARD = 0x69
	CHANNEL_SOUND = 0x67

	.macro EXOS n
		rst 0x30
		.DB n
	.endm

	; EXOS header "5"
	; linker will assume the code starts at 0xF0
	; thus the first code byte will be at 0x100 as
	; the header is 16 bytes long
	.DB 0,5
	.DW _memtop_prg - 0x100 ; number of bytes of the program to read
	.DB 0,0,0,0,0,0,0,0,0,0,0,0

;	; TODO: what happens, if program + stack, etc larger to
;	; fit into the first 16K page?
;	; first, init SP to 0x100, we need it before setting the good value
;	ld	sp,#0x100
;	; check if the total size we need exceeds 16K
;	ld	a, #(_memtop_inmem + STACK_SIZE)>>8
;	cp	a, #0x40

	ld	sp,#_memtop_inmem + STACK_SIZE
	ld	a,#12
	out	(191),a

	;call	gsinit ; this is needed for the ugly sdcc init data stuff :(

	;; **** SET VIDEO PARMS, OPEN VIDEO

	; for EXOS vars:
	; http://enterprise.iko.hu/variables.htm

	ld	bc,#0x100 + 22 ; 22[MODE_VID]
	ld	d,#MODE_VID
	rst	0x30
	.db	16
	;EXOS	16

	ld	bc,#0x100 + 23 ; 23[COLR_VID]
	ld	d,#COLR_VID
	rst	0x30
	.db	16
	;EXOS	16

	ld	bc,#0x100 + 24 ; 24[X_SIZ_VID]
	ld	d,#X_SIZ_VID
	rst	0x30
	.db 16
	;EXOS	16

	ld	bc,#0x100 + 25 ; 25[Y_SIZ_VID]
	ld	d,#Y_SIZ_VID
	rst	0x30
	.db 16
	;EXOS	16

	ld	a, #CHANNEL_VIDEO
	ld	de,#_video_channel_name
	rst	0x30
	.db	2 ; EXOS, open/create channel
	;EXOS	2

	ld	a, #CHANNEL_VIDEO
	ld	bc, #0x0101 ; B=1 (sub-func), C=starting line
	ld	de, #Y_SIZ_VID*256 + 1
	rst	0x30
	.db	11 ; EXOS, special functions (B = sub-func)
	;EXOS	11

	; **** OPEN KEYBOARD CHANNEL

	ld	a, #CHANNEL_KEYBOARD
	ld	de,#_keyboard_channel_name
	rst	0x30
	.db	1
	;EXOS	1

	; **** OPEN SOUND CHANNEL

	ld	a, #CHANNEL_SOUND
	ld	de, #_sound_channel_name
	rst	0x30
	.db	1
	;EXOS	1

	; **** CALL main()

	call	_main

	; **** END of C program, print a msg, wait for a key, return to BASIC

	; print msg
	ld	a,#CHANNEL_VIDEO
	ld	de,#endmsg
	ld	bc,#endmsg_end - endmsg
	rst	0x30
	.db	8
	;EXOS	8
	
	call 	_ping ; some sound effect :-P
	
	; wait for key
	; we use getchar() here to save extra bytes
	call	_getchar

_return_to_basic: ; so it can be used as return_to_basic() from C if exported
	ld	de, #BASIC
	rst	0x30
	.db	26
	;EXOS	26
; TODO: what will happen without BASIC?
sos:
	out	(0x81),a
	inc	a
	jr	sos


	; THIS IS putchar() function for the C runtime
	.globl	_putchar ; C syms are underscored in assembler
_putchar:
	push	ix ; SDCC uses IX to access params in stack. you must save it
	ld	ix,#0 ; ... and use ix=sp since Z80 lacks of sp rel LD ops.
	add	ix,sp
	ld	a, 4 (ix) ; C params are on the stack from offs of 4.
	cp	a, #0xA ; \n in C should CR + LF (at least it's my opinion)
	ld	b,a
	ld	a,#CHANNEL_VIDEO
	Jr	NZ, not_nl
	rst	0x30
	.db	7
	;EXOS	7
	ld	b, #0xD
	ld	a,#CHANNEL_VIDEO
not_nl:
	rst	0x30
	.db	7
	;EXOS	7
	pop	ix ; restore caller function's IX
	ret


	; THIS IS getchar() function for the C runtime
	; the same general rules as with putchar()
	.globl _getchar
_getchar:
	; TODO: will exos call destroy IX? if no, push/pop is useless!
	push	ix
	ld	a,#CHANNEL_KEYBOARD
	rst	0x30
	.db	5
	;EXOS	5
	ld	l,b  ; sdcc uses L to return byte-width retval of a C func
	pop	ix
	ret


_ping:
	ld	a,#CHANNEL_SOUND
	ld	b,#7
	rst	0x30
	.db	7
	;EXOS	7
	ret


_video_channel_name:
	.DB 6
	.ASCII "VIDEO:"
_keyboard_channel_name:
	.DB 9
	.ASCII "KEYBOARD:"
_sound_channel_name:
	.DB 6
	.ASCII "SOUND:"
BASIC:
	.DB 5
	.ASCII "BASIC"
endmsg:
	.DB 10,13
	.ASCII "Exit. Any key to BASIC."
endmsg_end:


	; These makes sure that segments will by linked in the right
	; order at the linking stage.
	.area	_HOME
	.area	_CODE
	.area	_CONST
        .area   _GSINIT
        .area   _GSFINAL
	.area	_INITEDDATA
	.area   _MEMTOP_PRG
	.area	_DATA
	;.area	_MEMTOP_PRG
	.area	_BSEG
        .area   _BSS
        .area   _HEAP
	.area   _OVERLAY
	.area	_MEMTOP_INMEM

	;; GSINIT stuffs
	;; We should make sdcc somehow not to emit code
	;; for initialized data but the data itself.
	;; Anyway till now, this will work.
;	.area	_GSINIT
;gsinit::
;	.area	_GSFINAL
;	ret

	;; MEMTOP is used to know the last byte of the program,
	;; for the EXOS header.
	.area	_MEMTOP_PRG
_memtop_prg::
	.area	_MEMTOP_INMEM
_memtop_inmem::


